/*================================================================================

	[M134EX: Weapon for Hero Survival]
	Copyright (C) 2014 by Nax0ne, Puerto Montt, Chile.
	Original code by LARS-DAY[BR]EAKER

	~~~~~~~~~~~
	- Licence -
	~~~~~~~~~~~
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	In addition, as a special exception, the author gives permission to
	link the code of this program with the Half-Life Game Engine ("HL
	Engine") and Modified Game Libraries ("MODs") developed by Valve,
	L.L.C ("Valve"). You must obey the GNU General Public License in all
	respects for all of the code used other than the HL Engine and MODs
	from Valve. If you modify this file, you may extend this exception
	to your version of the file, but you are not obligated to do so. If
	you do not wish to do so, delete this exception statement from your
	version.

=================================================================================*/

/*================================================================================
 [Include Files]
=================================================================================*/

#include < amxmodx >
#include < amxmisc >
#include < engine >
#include < fakemeta >
#include < fun >
#include < hamsandwich >
#include < xs >
#include < cstrike >
#include < zombie_scenario >

/*================================================================================
 [Vars, Arrays, CVARS Pointers & Bools]
=================================================================================*/

#define DAMAGE 5.25
#define RECOIL 2.5
#define CLIP 200
#define AMMO 400
#define SPEED 0.75
#define SPEED_FIRE 150.0
#define SPEED_RUN 200.0
#define SHELLS_LIFE 5000.0

#define ENG_NULLENT			-1
#define EV_INT_WEAPONKEY	EV_INT_impulse
#define m134ex_WEAPONKEY 	8567
#define MAX_PLAYERS  		32
#define IsValidUser(%1) (1 <= %1 <= g_MaxPlayers)

const USE_STOPPED = 0
const OFFSET_ACTIVE_ITEM = 373
const OFFSET_WEAPONOWNER = 41
const OFFSET_LINUX = 5
const OFFSET_LINUX_WEAPONS = 4

#define WEAP_LINUX_XTRA_OFF		4
#define m_fKnown					44
#define m_flNextPrimaryAttack 		46
#define m_flTimeWeaponIdle			48
#define m_iClip					51
#define m_fInReload				54
#define PLAYER_LINUX_XTRA_OFF	5
#define m_flNextAttack				83

#define m134ex_RELOAD_TIME	5.0
#define m134ex_DRAW_TIME 	1.1
#define m134ex_SHOOT1		1
#define m134ex_SHOOT2		2
#define m134ex_RELOAD		3
#define m134ex_DRAW			4
#define m134ex_FIRE_READY	5
#define m134ex_FIRE_AFTER	6

#define write_coord_f(%1)	engfunc(EngFunc_WriteCoord,%1)

new const Fire_Sounds[][] = { "weapons/m134ex-1.wav" }
new const g_shellent [] = "m134ex_shell"

new m134ex_V_MODEL[64] = "models/zombie_scenario/weapons/v_m134ex.mdl"
new m134ex_P_MODEL[64] = "models/zombie_scenario/weapons/p_m134ex.mdl"
new m134ex_W_MODEL[64] = "models/zombie_scenario/weapons/w_m134ex.mdl"

new const GUNSHOT_DECALS[] = { 41, 42, 43, 44, 45 }

new g_MaxPlayers, g_orig_event_m134ex, g_IsInPrimaryAttack
new Float:cl_pushangle[MAX_PLAYERS + 1][3], m_iBlood[2]
new g_has_m134ex[33], g_clip_ammo[33], g_m134ex_TmpClip[33], oldweap[33], g_can[33], g_afterreload[33]
new shell_mode, shell_mode2

new const WEAPONENTNAMES[][] = { "", "weapon_p228", "", "weapon_scout", "weapon_hegrenade", "weapon_xm1014", "weapon_c4", "weapon_mac10",
			"weapon_aug", "weapon_smokegrenade", "weapon_elite", "weapon_fiveseven", "weapon_ump45", "weapon_sg550",
			"weapon_galil", "weapon_famas", "weapon_usp", "weapon_glock18", "weapon_awp", "weapon_mp5navy", "weapon_m249",
			"weapon_m3", "weapon_m4a1", "weapon_tmp", "weapon_g3sg1", "weapon_flashbang", "weapon_deagle", "weapon_sg552",
			"weapon_ak47", "weapon_knife", "weapon_p90" }

const PRIMARY_WEAPONS_BIT_SUM = (1<<CSW_SCOUT)|(1<<CSW_XM1014)|(1<<CSW_MAC10)|(1<<CSW_AUG)|(1<<CSW_UMP45)|(1<<CSW_SG550)|(1<<CSW_GALIL)|(1<<CSW_FAMAS)|(1<<CSW_AWP)|(1<<CSW_MP5NAVY)|(1<<CSW_M249)|(1<<CSW_M3)|(1<<CSW_M4A1)|(1<<CSW_TMP)|(1<<CSW_G3SG1)|(1<<CSW_SG552)|(1<<CSW_AK47)|(1<<CSW_P90)
const SECONDARY_WEAPONS_BIT_SUM = (1<<CSW_P228)|(1<<CSW_ELITE)|(1<<CSW_FIVESEVEN)|(1<<CSW_USP)|(1<<CSW_GLOCK18)|(1<<CSW_DEAGLE)

/*================================================================================
 [Plugin Init & Precache]
=================================================================================*/

public plugin_init()
{	
	register_message(get_user_msgid("DeathMsg"), "message_DeathMsg");
	register_event("HLTV", "RoundStart_Pre", "a", "1=0", "2=0");
	register_event("CurWeapon","CurrentWeapon","be","1=1");
	RegisterHam(Ham_Item_AddToPlayer, "weapon_m249", "fw_m134ex_AddToPlayer");
	
	for(new i = 1; i < sizeof WEAPONENTNAMES; i++)
	{
		if (WEAPONENTNAMES[i][0]) RegisterHam(Ham_Item_Deploy, WEAPONENTNAMES[i], "fw_Item_Deploy_Post", 1);
	}
	
	RegisterHam(Ham_Weapon_PrimaryAttack, "weapon_m249", "fw_m134ex_PrimaryAttack");
	RegisterHam(Ham_Weapon_PrimaryAttack, "weapon_m249", "fw_m134ex_PrimaryAttack_Post", 1);
	RegisterHam(Ham_Item_PostFrame, "weapon_m249", "m134ex_ItemPostFrame");
	RegisterHam(Ham_Weapon_Reload, "weapon_m249", "m134ex_Reload");
	RegisterHam(Ham_Weapon_Reload, "weapon_m249", "m134ex_Reload_Post", 1);
	RegisterHam(Ham_TakeDamage, "info_target", "fw_TakeDamage");
	register_forward(FM_SetModel, "fw_SetModel");
	register_forward(FM_UpdateClientData, "fw_UpdateClientData_Post", 1);
	register_forward(FM_PlaybackEvent, "fwPlaybackEvent");
	register_forward(FM_CmdStart, "fw_CmdStart");
	register_forward(FM_PlayerPreThink, "fw_PlayerPreThink");
	register_think(g_shellent,"think_shell");
	
	RegisterHam(Ham_TraceAttack, "worldspawn", "fw_TraceAttack", 1);
	RegisterHam(Ham_TraceAttack, "info_target", "fw_TraceAttack", 1);
	
	g_MaxPlayers = get_maxplayers()
}

public plugin_precache()
{
	precache_model(m134ex_V_MODEL)
	precache_model(m134ex_P_MODEL)
	precache_model(m134ex_W_MODEL)
	
	for(new i = 0; i < sizeof Fire_Sounds; i++)
	{
		precache_sound(Fire_Sounds[i])
	}
	
	precache_sound("weapons/m134ex_clipoff.wav")
	precache_sound("weapons/m134ex_clipon.wav")
	precache_sound("weapons/m134ex_pinpull.wav")
  	precache_sound("weapons/m134ex_spindown.wav") 
  	precache_sound("weapons/m134ex_spinup.wav") 
	shell_mode = precache_model("models/zombie_scenario/weapons/shell.mdl")
	shell_mode2 = precache_model("models/zombie_scenario/weapons/shell2.mdl")
	m_iBlood[0] = precache_model("sprites/blood.spr")
	m_iBlood[1] = precache_model("sprites/bloodspray.spr")
	register_clcmd("weapon_m134ex", "weapon_hook")
	register_forward(FM_PrecacheEvent, "fwPrecacheEvent_Post", 1)
}
	
/*================================================================================
 [Main Events & Forwards]
=================================================================================*/

public weapon_hook(id) engclient_cmd(id, "weapon_m249")

public Get_Item_Hero(id)
{
	drop_weapons(id, 1);
	
	new iWep2 = give_item(id,"weapon_m249")
	
	if( iWep2 > 0 )
	{
		cs_set_weapon_ammo(iWep2, CLIP)
		cs_set_user_bpammo (id, CSW_M249, AMMO)	
		UTIL_PlayWeaponAnimation(id, m134ex_DRAW)
		set_pdata_float(id, m_flNextAttack, m134ex_DRAW_TIME, PLAYER_LINUX_XTRA_OFF)
	}
	
	g_has_m134ex[id] = true
}	

public fwPrecacheEvent_Post(type, const name[])
{
	if (equal("events/m249.sc", name))
	{
		g_orig_event_m134ex = get_orig_retval()
		return FMRES_HANDLED
	}
	return FMRES_IGNORED
}

public client_PreThink(id)
{
	if(!is_user_alive(id) || !g_has_m134ex[id] || get_user_weapon(id) != CSW_M249)
		return;

	new szClip, szAmmo
	get_user_weapon(id, szClip, szAmmo)

	if(szClip <= 0 && szAmmo <= 0)
		g_can[id] = 0

	set_hudmessage(200, 80, 0, 1.0, 0.98, 0, 0.1, 0.1,0.0,0.0)
	show_hudmessage(id, "Ammo: %d / %d",szClip,szAmmo)
}

public client_connect(id)
{
	g_has_m134ex[id] = false
}

public client_disconnect(id)
{
	g_has_m134ex[id] = false
}

public RoundStart_Pre(id)
{
	g_has_m134ex[id] = false
}

public CurrentWeapon(id)
{
     replace_weapon_models(id, read_data(2))

     if(read_data(2) != CSW_M249 || !g_has_m134ex[id])
          return
     
     static Float:iSpeed
     if(g_has_m134ex[id])
          iSpeed = SPEED
     
     static weapon[32],Ent
     get_weaponname(read_data(2),weapon,31)
     Ent = find_ent_by_owner(-1,weapon,id)
     if(Ent)
     {
          static Float:Delay
          Delay = get_pdata_float( Ent, 46, 4) * iSpeed
          if (Delay > 0.0)
          {
               set_pdata_float(Ent, 46, Delay, 4)
          }
     }
}

public fw_TraceAttack(iEnt, iAttacker, Float:flDamage, Float:fDir[3], ptr, iDamageType)
{
	if(!is_user_alive(iAttacker))
		return

	new g_currentweapon = get_user_weapon(iAttacker)

	if(g_currentweapon != CSW_M249) return
	
	if(!g_has_m134ex[iAttacker]) return

	static Float:flEnd[3]
	get_tr2(ptr, TR_vecEndPos, flEnd)
	
	if(iEnt)
	{
		message_begin(MSG_BROADCAST, SVC_TEMPENTITY)
		write_byte(TE_DECAL)
		write_coord_f(flEnd[0])
		write_coord_f(flEnd[1])
		write_coord_f(flEnd[2])
		write_byte(GUNSHOT_DECALS[random_num (0, sizeof GUNSHOT_DECALS -1)])
		write_short(iEnt)
		message_end()
	}
	else
	{
		message_begin(MSG_BROADCAST, SVC_TEMPENTITY)
		write_byte(TE_WORLDDECAL)
		write_coord_f(flEnd[0])
		write_coord_f(flEnd[1])
		write_coord_f(flEnd[2])
		write_byte(GUNSHOT_DECALS[random_num (0, sizeof GUNSHOT_DECALS -1)])
		message_end()
	}
	
	message_begin(MSG_BROADCAST, SVC_TEMPENTITY)
	write_byte(TE_GUNSHOTDECAL)
	write_coord_f(flEnd[0])
	write_coord_f(flEnd[1])
	write_coord_f(flEnd[2])
	write_short(iAttacker)
	write_byte(GUNSHOT_DECALS[random_num (0, sizeof GUNSHOT_DECALS -1)])
	message_end()
}

public fw_SetModel(entity, model[])
{
	if(!is_valid_ent(entity))
		return FMRES_IGNORED
	
	static szClassName[33]
	entity_get_string(entity, EV_SZ_classname, szClassName, charsmax(szClassName))
		
	if(!equal(szClassName, "weaponbox"))
		return FMRES_IGNORED
	
	static iOwner
	
	iOwner = entity_get_edict(entity, EV_ENT_owner)
	
	if(equal(model, "models/w_m249.mdl"))
	{
		static iStoredAugID
		
		iStoredAugID = find_ent_by_owner(ENG_NULLENT, "weapon_m249", entity)
	
		if(!is_valid_ent(iStoredAugID))
			return FMRES_IGNORED
	
		if(g_has_m134ex[iOwner])
		{
			entity_set_int(iStoredAugID, EV_INT_WEAPONKEY, m134ex_WEAPONKEY)
			
			g_has_m134ex[iOwner] = false
			
			entity_set_model(entity, m134ex_W_MODEL)
			
			return FMRES_SUPERCEDE
		}
	}
	return FMRES_IGNORED
}

public fw_m134ex_AddToPlayer(m134ex, id)
{
	if(!is_valid_ent(m134ex) || !is_user_connected(id))
		return HAM_IGNORED
	
	if(entity_get_int(m134ex, EV_INT_WEAPONKEY) == m134ex_WEAPONKEY)
	{
		g_has_m134ex[id] = true
		
		entity_set_int(m134ex, EV_INT_WEAPONKEY, 0)
		
		return HAM_HANDLED
	}
	return HAM_IGNORED
}

public fw_Item_Deploy_Post(weapon_ent)
{
	static owner
	owner = fm_cs_get_weapon_ent_owner(weapon_ent)
	
	static weaponid
	weaponid = cs_get_weapon_id(weapon_ent)
	
	replace_weapon_models(owner, weaponid)
}

public fw_UpdateClientData_Post(Player, SendWeapons, CD_Handle)
{
	if(!is_user_alive(Player) || (get_user_weapon(Player) != CSW_M249 || !g_has_m134ex[Player]))
		return FMRES_IGNORED
	
	set_cd(CD_Handle, CD_flNextAttack, halflife_time () + 0.001)
	return FMRES_HANDLED
}

public fw_CmdStart(id, uc_handle, seed)
{
	if(!is_user_alive(id) || get_user_weapon(id) != CSW_M249 || !g_has_m134ex[id]) 
		return PLUGIN_HANDLED

	new Float:flNextAttack = get_pdata_float(id, m_flNextAttack, PLAYER_LINUX_XTRA_OFF)

	if(flNextAttack > 0.0)
		return PLUGIN_HANDLED

	new szClip, szAmmo
	get_user_weapon(id, szClip, szAmmo)

	if(szClip <= 0)
		return PLUGIN_HANDLED

	if(!(pev(id, pev_oldbuttons) & IN_ATTACK))
	{
		remove_task(id)
		set_pdata_float(id, m_flNextAttack, 0.0, PLAYER_LINUX_XTRA_OFF)
		g_can[id] = 0
	}
	if((pev(id, pev_oldbuttons) & IN_ATTACK) && !(get_uc(uc_handle, UC_Buttons) & IN_ATTACK))
	{
		remove_task(id)
		set_task(1.0,"user_can2",id)
		g_can[id] = 3
		set_pdata_float(id, m_flNextAttack, m134ex_DRAW_TIME, PLAYER_LINUX_XTRA_OFF)
		UTIL_PlayWeaponAnimation(id,m134ex_FIRE_AFTER)
		g_can[id] = 0
	}
	return PLUGIN_HANDLED
}

public fw_m134ex_PrimaryAttack(Weapon)
{
	new Player = get_pdata_cbase(Weapon, 41, 4)
	
	if (!g_has_m134ex[Player])
		return HAM_IGNORED

	g_IsInPrimaryAttack = 1

	if(g_afterreload[Player])
	{
		g_afterreload[Player] = 0
		return HAM_SUPERCEDE
	}
	new Float:flNextAttack = get_pdata_float(Player, m_flNextAttack, PLAYER_LINUX_XTRA_OFF)

	if(flNextAttack > 0.0)
		return HAM_IGNORED

	new szClip, szAmmo
	get_user_weapon(Player, szClip, szAmmo)

	if(!g_can[Player] || g_can[Player] == 3)
	{
		if(szClip <= 0)
		{
			UTIL_PlayWeaponAnimation(Player,m134ex_FIRE_AFTER)
			set_pdata_float(Player, m_flNextAttack, m134ex_DRAW_TIME, PLAYER_LINUX_XTRA_OFF)
			return HAM_SUPERCEDE
		}
		set_task(1.0,"user_can",Player)
		g_can[Player] = 1
		set_pdata_float(Player, m_flNextAttack, 1.1, PLAYER_LINUX_XTRA_OFF)
		UTIL_PlayWeaponAnimation(Player,m134ex_FIRE_READY)
		return HAM_SUPERCEDE
	}
	pev(Player,pev_punchangle,cl_pushangle[Player])
	
	g_clip_ammo[Player] = cs_get_weapon_ammo(Weapon)

	return HAM_IGNORED
}

public fwPlaybackEvent(flags, invoker, eventid, Float:delay, Float:origin[3], Float:angles[3], Float:fparam1, Float:fparam2, iParam1, iParam2, bParam1, bParam2)
{
	if ((eventid != g_orig_event_m134ex) || !g_IsInPrimaryAttack)
		return FMRES_IGNORED
	if (!(1 <= invoker <= g_MaxPlayers))
    return FMRES_IGNORED

	playback_event(flags | FEV_HOSTONLY, invoker, eventid, delay, origin, angles, fparam1, fparam2, iParam1, iParam2, bParam1, bParam2)
	return FMRES_SUPERCEDE
}

public fw_m134ex_PrimaryAttack_Post(Weapon)
{
	g_IsInPrimaryAttack = 0
	new Player = get_pdata_cbase(Weapon, 41, 4)
	
	if(!is_user_alive(Player))
		return

	new szClip, szAmmo
	get_user_weapon(Player, szClip, szAmmo)

	if(g_has_m134ex[Player])
	{
		if(g_can[Player] != 2)
			return

		if(szClip <= 0)
		{
			UTIL_PlayWeaponAnimation(Player,m134ex_FIRE_AFTER)
			set_pdata_float(Player, m_flNextAttack, m134ex_DRAW_TIME, PLAYER_LINUX_XTRA_OFF)
		}

		if (!g_clip_ammo[Player])
			return
		
		static Float:g_punchangle[3]
		g_punchangle[0] = -1 * RECOIL
		set_pev(Player, pev_punchangle, g_punchangle); 
		
		emit_sound(Player, CHAN_WEAPON, Fire_Sounds[0], VOL_NORM, ATTN_NORM, 0, PITCH_NORM)
		UTIL_PlayWeaponAnimation(Player, random_num(m134ex_SHOOT1,m134ex_SHOOT2))
		//make_shell(Player)
	}
}

public fw_TakeDamage(victim, inflictor, attacker, Float:damage)
{
	if (victim != attacker && is_user_connected(attacker))
	{
		if(get_user_weapon(attacker) == CSW_M249)
		{
			if(g_has_m134ex[attacker])
			{
				SetHamParamFloat(4, damage * DAMAGE)
			}
		}
	}
}

public m134ex_ItemPostFrame(weapon_entity) 
{
     new id = pev(weapon_entity, pev_owner)
     if (!is_user_connected(id))
          return HAM_IGNORED

     if (!g_has_m134ex[id])
          return HAM_IGNORED

     new Float:flNextAttack = get_pdata_float(id, m_flNextAttack, PLAYER_LINUX_XTRA_OFF)

     new iBpAmmo = cs_get_user_bpammo(id, CSW_M249)
     new iClip = get_pdata_int(weapon_entity, m_iClip, WEAP_LINUX_XTRA_OFF)

     new fInReload = get_pdata_int(weapon_entity, m_fInReload, WEAP_LINUX_XTRA_OFF) 

     if( fInReload && flNextAttack <= 0.0 )
     {
	     new j = min(CLIP - iClip, iBpAmmo)
	
	     set_pdata_int(weapon_entity, m_iClip, iClip + j, WEAP_LINUX_XTRA_OFF)
	     cs_set_user_bpammo(id, CSW_M249, iBpAmmo-j)
		
	     set_pdata_int(weapon_entity, m_fInReload, 0, WEAP_LINUX_XTRA_OFF)
	     fInReload = 0
	     g_can[id] = 0
     }
     return HAM_IGNORED
}

public m134ex_Reload(weapon_entity) 
{
     new id = pev(weapon_entity, pev_owner)
     if (!is_user_connected(id))
          return HAM_IGNORED

     if (!g_has_m134ex[id])
          return HAM_IGNORED

     g_m134ex_TmpClip[id] = -1

     new iBpAmmo = cs_get_user_bpammo(id, CSW_M249)
     new iClip = get_pdata_int(weapon_entity, m_iClip, WEAP_LINUX_XTRA_OFF)

     if (iBpAmmo <= 0)
          return HAM_SUPERCEDE

     if (iClip >= CLIP)
          return HAM_SUPERCEDE

     g_m134ex_TmpClip[id] = iClip

     return HAM_IGNORED
}

public m134ex_Reload_Post(weapon_entity) 
{
	new id = pev(weapon_entity, pev_owner)
	if (!is_user_connected(id))
		return HAM_IGNORED

	if (!g_has_m134ex[id])
		return HAM_IGNORED

	new szClip, szAmmo
	get_user_weapon(id, szClip, szAmmo)

	if (szClip == CLIP)
		return HAM_IGNORED

	if (szAmmo <= 0)
		return HAM_IGNORED

	if (g_m134ex_TmpClip[id] == -1)
		return HAM_IGNORED

	set_pdata_int(weapon_entity, m_iClip, g_m134ex_TmpClip[id], WEAP_LINUX_XTRA_OFF)

	set_pdata_float(weapon_entity, m_flTimeWeaponIdle, m134ex_RELOAD_TIME, WEAP_LINUX_XTRA_OFF)

	set_pdata_float(id, m_flNextAttack, m134ex_RELOAD_TIME, PLAYER_LINUX_XTRA_OFF)

	set_pdata_int(weapon_entity, m_fInReload, 1, WEAP_LINUX_XTRA_OFF)

	UTIL_PlayWeaponAnimation(id, m134ex_RELOAD)

	remove_task(id)
	g_can[id] = 0
	g_afterreload[id] = 1

	return HAM_IGNORED
}

public fw_PlayerPreThink(id)
{
	if(!is_user_alive(id) || !g_has_m134ex[id] || get_user_weapon(id) != CSW_M249)
		return

	new szClip
	get_user_weapon(id, szClip)

	if(g_can[id]) set_pev(id, pev_maxspeed, SPEED_FIRE)
	if(!g_can[id] || szClip <= 0) set_pev(id, pev_maxspeed, SPEED_RUN)
}

public message_DeathMsg(msg_id, msg_dest, id)
{
	static szTruncatedWeapon[33], iAttacker, iVictim
	
	get_msg_arg_string(4, szTruncatedWeapon, charsmax(szTruncatedWeapon))
	
	iAttacker = get_msg_arg_int(1)
	iVictim = get_msg_arg_int(2)
	
	if(!is_user_connected(iAttacker) || iAttacker == iVictim)
		return PLUGIN_CONTINUE
	
	if(equal(szTruncatedWeapon, "m249") && get_user_weapon(iAttacker) == CSW_M249)
	{
		if(g_has_m134ex[iAttacker])
			set_msg_arg_string(4, "m249")
	}
	return PLUGIN_CONTINUE
}

public user_can(id) 
{
	g_can[id] = 2
}

public user_can2(id) 
{
	g_can[id] = 0
}

public make_shell(id)
{
	static Float:origin[3], Float:origin2[3],  Float:vSrc[3], Float:angles[3], Float:v_forward[3], Float:v_right[3], Float:v_up[3], Float:gun_position[3], Float:player_origin[3], Float:player_view_offset[3]
	static Float:OriginX[3]
	pev(id, pev_v_angle, angles)
	pev(id, pev_origin, OriginX)
	engfunc(EngFunc_MakeVectors, angles)

	static Float:v_forward2[3], Float:v_right2[3], Float:v_up2[3], Float:vSrc2[3]

	global_get(glb_v_forward, v_forward)
	global_get(glb_v_right, v_right)
	global_get(glb_v_up, v_up)

	global_get(glb_v_forward, v_forward2)
	global_get(glb_v_right, v_right2)
	global_get(glb_v_up, v_up2)

	//m_pPlayer->GetGunPosition( ) = pev->origin + pev->view_ofs
	pev(id, pev_origin, player_origin)
	pev(id, pev_view_ofs, player_view_offset)
	xs_vec_add(player_origin, player_view_offset, gun_position)

	xs_vec_mul_scalar(v_forward, 10.0, v_forward)
	xs_vec_mul_scalar(v_right, 3.0, v_right)
	xs_vec_mul_scalar(v_up, -8.0, v_up)

	xs_vec_mul_scalar(v_forward2, random_float(0.01, 10.0), v_forward2)
	xs_vec_mul_scalar(v_right2, random_float(0.0, -2.5), v_right2)
	xs_vec_mul_scalar(v_up2,-30.0, v_up2)

	xs_vec_add(gun_position, v_forward, origin)
	xs_vec_add(origin, v_right, origin)
	xs_vec_add(origin, v_up, origin)

	xs_vec_add(gun_position, v_forward2, origin2)
	xs_vec_add(origin2, v_right2, origin2)
	xs_vec_add(origin2, v_up2, origin2)

	vSrc[0] = origin[0]
	vSrc[1] = origin[1]
	vSrc[2] = origin[2]

	vSrc2[0] = origin2[0]
	vSrc2[1] = origin2[1]
	vSrc2[2] = origin2[2]

	new ent = create_entity("info_target")
	set_pev(ent, pev_classname, g_shellent)

	engfunc(EngFunc_SetModel,ent, "models/zombie_scenario/weapons/shell.mdl")

	set_pev(ent,pev_mins,Float:{-1.0,-1.0,0.0})
	set_pev(ent,pev_maxs,Float:{1.0,1.0,1.0})
	set_pev(ent,pev_size,Float:{-1.0,-1.0,0.0,1.0,1.0,1.0})
	engfunc(EngFunc_SetSize,ent,Float:{-1.0,-1.0,0.0},Float:{1.0,1.0,1.0})

	//set_pev(ent,pev_solid,SOLID_BBOX)
	//set_pev(ent,pev_movetype,MOVETYPE_TOSS)

	set_pev(ent, pev_origin, vSrc)

	static Float:newangles[3]
	pev(id,pev_angles,newangles)
	set_pev(ent,pev_angles, newangles)

	static Float:flVelocity [ 3 ]
	get_speed_vector ( vSrc2, vSrc, random_float(100.0,150.0), flVelocity )
	set_pev(ent,pev_velocity,flVelocity)

	entity_set_float(ent, EV_FL_health, SHELLS_LIFE)

	entity_set_float(ent, EV_FL_nextthink, get_gametime() + 0.01) 
	
	new angle = random_num(0,360)
	
	message_begin(MSG_ONE_UNRELIABLE, SVC_TEMPENTITY,_,id)
	write_byte(TE_MODEL)
	write_coord_f(OriginX[0])
	write_coord_f(OriginX[1])
	write_coord_f(OriginX[2])
	write_coord_f(flVelocity[0])
	write_coord_f(flVelocity[1])
	write_coord_f(flVelocity[2])
	write_angle(angle)
	write_short(shell_mode)
	write_byte(1)
	write_byte(floatround(SHELLS_LIFE) * 10)
	message_end()

	make_shell2(id)
}

public make_shell2(id)
{
	static Float:origin[3], Float:origin2[3],  Float:vSrc[3], Float:angles[3], Float:v_forward[3], Float:v_right[3], Float:v_up[3], Float:gun_position[3], Float:player_origin[3], Float:player_view_offset[3]
	static Float:OriginX[3]
	pev(id, pev_v_angle, angles)
	pev(id, pev_origin, OriginX)
	engfunc(EngFunc_MakeVectors, angles)

	static Float:v_forward2[3] , Float:v_right2[3],  Float:v_up2[3] , Float:vSrc2[3]

	global_get(glb_v_forward, v_forward)
	global_get(glb_v_right, v_right)
	global_get(glb_v_up, v_up)

	global_get(glb_v_forward, v_forward2)
	global_get(glb_v_right, v_right2)
	global_get(glb_v_up, v_up2)

	//m_pPlayer->GetGunPosition( ) = pev->origin + pev->view_ofs
	pev(id, pev_origin, player_origin)
	pev(id, pev_view_ofs, player_view_offset)
	xs_vec_add(player_origin, player_view_offset, gun_position)

	xs_vec_mul_scalar(v_forward, 13.0, v_forward);
	xs_vec_mul_scalar(v_right, random_float(-4.5,-6.0), v_right);
	xs_vec_mul_scalar(v_up, -5.0, v_up); // -3.0

	xs_vec_mul_scalar(v_forward2, 0.0, v_forward2);
	xs_vec_mul_scalar(v_right2, 0.0, v_right2);
	xs_vec_mul_scalar(v_up2, 0.0, v_up2);

	xs_vec_add(gun_position, v_forward, origin)
	xs_vec_add(origin, v_right, origin)
	xs_vec_add(origin, v_up, origin)

	xs_vec_add(gun_position, v_forward2, origin2)
	xs_vec_add(origin2, v_right2, origin2)
	xs_vec_add(origin2, v_up2, origin2)

	vSrc[0] = origin[0]
	vSrc[1] = origin[1]
	vSrc[2] = origin[2]

	vSrc2[0] = origin2[0]
	vSrc2[1] = origin2[1]
	vSrc2[2] = origin2[2]

	new ent = create_entity("info_target")
	set_pev(ent, pev_classname, g_shellent)

	engfunc(EngFunc_SetModel,ent, "models/zombie_scenario/weapons/shell2.mdl")

	set_pev(ent,pev_mins,Float:{-1.0,-1.0,0.0})
	set_pev(ent,pev_maxs,Float:{1.0,1.0,1.0})
	set_pev(ent,pev_size,Float:{-1.0,-1.0,0.0,1.0,1.0,1.0})
	engfunc(EngFunc_SetSize,ent,Float:{-1.0,-1.0,0.0},Float:{1.0,1.0,1.0})

	//set_pev(ent,pev_solid, SOLID_TRIGGER)
	//set_pev(ent,pev_movetype, MOVETYPE_TOSS)
	
	new angle = random_num(0,360)

	set_pev(ent, pev_origin, vSrc)

	static Float:newangles[3]
	pev(id,pev_angles,newangles)
	set_pev(ent,pev_angles, newangles)

	static Float:flVelocity [ 3 ]
	get_speed_vector ( vSrc2, vSrc, random_float(100.0, 200.0), flVelocity )
	set_pev(ent,pev_velocity,flVelocity)
	
	message_begin(MSG_ONE_UNRELIABLE, SVC_TEMPENTITY,_,id)
	write_byte(TE_MODEL)
	write_coord_f(OriginX[0])
	write_coord_f(OriginX[1])
	write_coord_f(OriginX[2])
	write_coord_f(flVelocity[0])
	write_coord_f(flVelocity[1])
	write_coord_f(flVelocity[2])
	write_angle(angle)
	write_short(shell_mode2)
	write_byte(1)
	write_byte(floatround(SHELLS_LIFE) * 10)
	message_end()

	entity_set_float(ent, EV_FL_health, SHELLS_LIFE)

	entity_set_float(ent, EV_FL_nextthink, get_gametime() + 0.01) 
}

public think_shell(ent) 
{
	if(!pev_valid(ent))
		return
		
	if(!(pev(ent,pev_flags) & FL_ONGROUND))
	{
		new Float:oldangles[3],Float:angles[3]
		pev(ent,pev_angles,oldangles)
		angles[0] = oldangles[0] + random_float(10.0,20.0)
		angles[1] = oldangles[1] + random_float(10.0,20.0)
		angles[2] = oldangles[2] + random_float(10.0,20.0)
		set_pev(ent,pev_angles,angles)
	}
	entity_set_float(ent, EV_FL_health, entity_get_float(ent,EV_FL_health) - 10.0) 
	entity_set_float(ent, EV_FL_nextthink, get_gametime() + 0.01) 
	if(entity_get_float(ent,EV_FL_health) <= 0) remove_entity(ent)
}

/*================================================================================
 [Internal Functions]
=================================================================================*/

replace_weapon_models(id, weaponid)
{
	switch (weaponid)
	{
		case CSW_M249:
		{			
			if(g_has_m134ex[id])
			{
				set_pev(id, pev_viewmodel2, m134ex_V_MODEL)

				set_pev(id, pev_weaponmodel2, m134ex_P_MODEL)
	
				message_begin(MSG_ONE, get_user_msgid("CurWeapon"), {0,0,0}, id) 
				write_byte(1) 
				write_byte(CSW_KNIFE) 
				write_byte(0) 
				message_end()

				if(oldweap[id] != CSW_M249) 
				{
					UTIL_PlayWeaponAnimation(id, m134ex_DRAW)
					set_pdata_float(id, m_flNextAttack, m134ex_DRAW_TIME, PLAYER_LINUX_XTRA_OFF)
				
				}
			}
		}
	}
	if(weaponid != CSW_M249)
	{
		remove_task(id)
		g_can[id] = 0
		g_afterreload[id] = 0
		if(oldweap[id] == CSW_M249 && g_has_m134ex[id]) 
		{
			message_begin(MSG_ONE, get_user_msgid("CurWeapon"), {0,0,0}, id) 
			write_byte(1) 
			write_byte(CSW_KNIFE) 
			write_byte(0) 
			message_end()
		}
	}
	oldweap[id] = weaponid
}

/*================================================================================
 [Stocks]
=================================================================================*/

stock drop_weapons(id, dropwhat)
{
	// Get user weapons
	static weapons[32], num, i, weaponid
	num = 0 // reset passed weapons count (bugfix)
	get_user_weapons(id, weapons, num)
	
	// Loop through them and drop primaries or secondaries
	for (i = 0; i < num; i++)
	{
		// Prevent re-indexing the array
		weaponid = weapons[i]
		
		if ((dropwhat == 1 && ((1<<weaponid) & PRIMARY_WEAPONS_BIT_SUM)) || (dropwhat == 2 && ((1<<weaponid) & SECONDARY_WEAPONS_BIT_SUM)))
		{
			// Get weapon entity
			static wname[32]; get_weaponname(weaponid, wname, charsmax(wname))
			engclient_cmd(id, "drop", wname)
		}
	}
}

stock get_speed_vector(const Float:origin1[3],const Float:origin2[3],Float:speed, Float:new_velocity[3])
{
	new_velocity[0] = origin2[0] - origin1[0]
	new_velocity[1] = origin2[1] - origin1[1]
	new_velocity[2] = origin2[2] - origin1[2]
	new Float:num = floatsqroot(speed*speed / (new_velocity[0]*new_velocity[0] + new_velocity[1]*new_velocity[1] + new_velocity[2]*new_velocity[2]))
	new_velocity[0] *= num
	new_velocity[1] *= num
	new_velocity[2] *= num
       
	return 1
} 

stock fm_cs_get_current_weapon_ent(id)
{
	return get_pdata_cbase(id, OFFSET_ACTIVE_ITEM, OFFSET_LINUX)
}

stock fm_cs_get_weapon_ent_owner(ent)
{
	return get_pdata_cbase(ent, OFFSET_WEAPONOWNER, OFFSET_LINUX_WEAPONS)
}

stock UTIL_PlayWeaponAnimation(const Player, const Sequence)
{
	set_pev(Player, pev_weaponanim, Sequence)
	
	message_begin(MSG_ONE_UNRELIABLE, SVC_WEAPONANIM, .player = Player)
	write_byte(Sequence)
	write_byte(pev(Player, pev_body))
	message_end()
}