/*================================================================================

	[Zombie Scenario Shop: Dual Infinity]
	Copyright (C) 2014 by Nax0ne, Puerto Montt, Chile.
	Original code by Dias Leon

	~~~~~~~~~~~
	- Licence -
	~~~~~~~~~~~
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
	
	In addition, as a special exception, the author gives permission to
	link the code of this program with the Half-Life Game Engine ("HL
	Engine") and Modified Game Libraries ("MODs") developed by Valve,
	L.L.C ("Valve"). You must obey the GNU General Public License in all
	respects for all of the code used other than the HL Engine and MODs
	from Valve. If you modify this file, you may extend this exception
	to your version of the file, but you are not obligated to do so. If
	you do not wish to do so, delete this exception statement from your
	version.

=================================================================================*/

/*================================================================================
 [Include Files]
=================================================================================*/

#include < amxmodx >
#include < engine >
#include < fakemeta >
#include < fakemeta_util >
#include < hamsandwich >
#include < cstrike >
#include < fun >
#include < zbs_shop >
#include < zombie_scenario >

/*================================================================================
 [Vars, Arrays, CVARS Pointers & Bools]
=================================================================================*/

// Item Info
new const ITEM_NAME[] = { "Dual Infinity" }
const ITEM_COST = 2750
const ITEM_LEVEL = 14

#define CSW_DUALINFINITY CSW_ELITE
#define weapon_dualinfinity "weapon_elite"

#define DI_CODE 423423

#define DI_DEFAULT_BPAMMO 200
#define DI_DEFAULT_CLIP 30
#define DI_DAMAGE_SEMI 30
#define DI_DAMAGE_RAPID 42
#define DI_RECOIL_SEMI 0.75
#define DI_RECOIL_RAPID 0.25

new const di_model[4][] =
{
	"models/zombie_scenario/weapons/v_infinityex2.mdl",
	"models/zombie_scenario/weapons/v_infinityex2_2.mdl",
	"models/zombie_scenario/weapons/p_infinityex2.mdl",
	"models/zombie_scenario/weapons/w_infinityex2.mdl"
}

new const di_sound[5][] =
{
	"weapons/infi-1.wav",
	"weapons/infi_clipin.wav",
	"weapons/infi_clipon.wav",
	"weapons/infi_clipout.wav",
	"weapons/infi_draw.wav"
}

enum
{
	MODE_SEMI = 0,
	MODE_RAPID,
	MODE_RAPID2
}

const m_szAnimExtention = 492

new g_had_dualinfinity[33], g_shoot_mode[33], g_dualinfinity_clip[33], Float:cl_pushangle[33][3]
new g_di_event_left, g_di_event_right, g_smokepuff_id, m_iBlood[2], Float:g_last_shoot[33], Float:AimOrigin[33][3]

const PRIMARY_WEAPONS_BIT_SUM = (1<<CSW_SCOUT)|(1<<CSW_XM1014)|(1<<CSW_MAC10)|(1<<CSW_AUG)|(1<<CSW_UMP45)|(1<<CSW_SG550)|(1<<CSW_GALIL)|(1<<CSW_FAMAS)|(1<<CSW_AWP)|(1<<CSW_MP5NAVY)|(1<<CSW_M249)|(1<<CSW_M3)|(1<<CSW_M4A1)|(1<<CSW_TMP)|(1<<CSW_G3SG1)|(1<<CSW_SG552)|(1<<CSW_AK47)|(1<<CSW_P90)
const SECONDARY_WEAPONS_BIT_SUM = (1<<CSW_P228)|(1<<CSW_ELITE)|(1<<CSW_FIVESEVEN)|(1<<CSW_USP)|(1<<CSW_GLOCK18)|(1<<CSW_DEAGLE)

/*================================================================================
 [Plugin Init + Precache]
=================================================================================*/

public plugin_init()
{
	// Register New Item
	zbs_shop_item_add(ITEM_NAME, ITEM_COST, ITEM_LEVEL, "handler_DualInfinity");
	
	// Register Events & Forwards
	register_event("CurWeapon", "Event_CheckWeapon", "be", "1=1");
	
	register_forward(FM_UpdateClientData, "fw_UpdateClientData_Post", 1);	
	register_forward(FM_PlaybackEvent, "fw_PlaybackEvent");	
	register_forward(FM_SetModel, "fw_SetModel");
	register_forward(FM_CmdStart, "fw_CmdStart");
	
	RegisterHam(Ham_TraceAttack, "worldspawn", "fw_TraceAttack");
	RegisterHam(Ham_TraceAttack, "info_target", "fw_TraceAttack");	
	RegisterHam(Ham_Item_AddToPlayer, weapon_dualinfinity, "fw_AddToPlayer_Post", 1);
	RegisterHam(Ham_Weapon_PrimaryAttack, weapon_dualinfinity, "fw_PrimaryAttack");
	RegisterHam(Ham_Weapon_PrimaryAttack, weapon_dualinfinity, "fw_PrimaryAttack_Post", 1);
	RegisterHam(Ham_Weapon_Reload, weapon_dualinfinity, "fw_Reload");
	RegisterHam(Ham_Weapon_Reload, weapon_dualinfinity, "fw_Reload_Post", 1);
	RegisterHam(Ham_Item_PostFrame, weapon_dualinfinity, "fw_PostFrame");
	//RegisterHam(Ham_Weapon_SendWeaponAnim, weapon_dualinfinity, "SendWeaponAnim", 1);
	
	// Client & Console CMDS
	register_clcmd("weapon_dualinfinity", "Hook_WeaponHud")
}

public plugin_precache()
{
	for(new i = 0; i < sizeof(di_model); i++)
		precache_model(di_model[i]);
	for(new i = 0; i < sizeof(di_sound); i++)
		precache_sound(di_sound[i]);
	
	g_smokepuff_id = precache_model("sprites/wall_puff1.spr")
	m_iBlood[0] = precache_model("sprites/blood.spr")
	m_iBlood[1] = precache_model("sprites/bloodspray.spr")	
	
	register_forward(FM_PrecacheEvent, "fw_PrecacheEvent_Post", 1)
}

/*================================================================================
 [Buy Item]
=================================================================================*/

public handler_DualInfinity(iPlayer)
{	
	Get_DualInfinity(iPlayer)
}

/*================================================================================
 [Main Events & Forwards]
=================================================================================*/

public fw_PrecacheEvent_Post(type, const name[])
{
	if(equal("events/elite_left.sc", name))
		g_di_event_left = get_orig_retval()
	if(equal("events/elite_right.sc", name))
		g_di_event_right = get_orig_retval()		
}

public Get_DualInfinity(id)
{
	if(!is_user_alive(id))
		return;
		
	drop_weapons(id, 2)	
		
	g_had_dualinfinity[id] = 1
	g_shoot_mode[id] = MODE_SEMI
	
	give_item(id, weapon_dualinfinity)
	cs_set_user_bpammo(id, CSW_DUALINFINITY, DI_DEFAULT_BPAMMO)
	
	static ent
	ent = fm_get_user_weapon_entity(id, CSW_DUALINFINITY)
	
	if(pev_valid(ent)) cs_set_weapon_ammo(ent, DI_DEFAULT_CLIP)
}

public Event_CheckWeapon(id)
{
	if(!is_user_alive(id))
		return 1
	if(get_user_weapon(id) != CSW_DUALINFINITY || !g_had_dualinfinity[id])
		return 1
		
	set_pev(id, pev_viewmodel2, g_shoot_mode[id] == MODE_SEMI ? di_model[0] : di_model[1])
	set_pev(id, pev_weaponmodel2, di_model[2])
		
	return 0
}

public fw_UpdateClientData_Post(id, sendweapons, cd_handle)
{
	if(!is_user_alive(id) || !is_user_connected(id))
		return FMRES_IGNORED;
	if(get_user_weapon(id) == CSW_DUALINFINITY && g_had_dualinfinity[id])
		set_cd(cd_handle, CD_flNextAttack, get_gametime() + 0.001) 
	
	return FMRES_HANDLED
}

public fw_PlaybackEvent(flags, invoker, eventid, Float:delay, Float:origin[3], Float:angles[3], Float:fparam1, Float:fparam2, iParam1, iParam2, bParam1, bParam2)
{
	if (!(1 <= invoker <= get_maxplayers()))
		return FMRES_IGNORED
	if(get_user_weapon(invoker) != CSW_DUALINFINITY || !g_had_dualinfinity[invoker])
		return FMRES_IGNORED
	
	if(eventid == g_di_event_left)
	{
		playback_event(flags | FEV_HOSTONLY, invoker, eventid, delay, origin, angles, fparam1, fparam2, iParam1, iParam2, bParam1, bParam2)
		
		set_weapon_anim(invoker, random_num(2, 6))
		emit_sound(invoker, CHAN_WEAPON, di_sound[0], 1.0, ATTN_NORM, 0, PITCH_NORM)
		
		static Float:Push[3]
		if(g_shoot_mode[invoker] == MODE_SEMI || g_shoot_mode[invoker] == MODE_RAPID2)
		{
			Push[0] = random_float(-1.0, 1.5)
			Push[1] = random_float(0.0, 1.0)
			Push[2] = 0.0
		} else if(g_shoot_mode[invoker] == MODE_RAPID) {
			Push[0] = random_float(0.0, 0.0)
			Push[1] = random_float(0.0, 0.0)
			Push[2] = 0.0			
		}
			
		set_pev(invoker, pev_punchangle, Push)
		
		return FMRES_SUPERCEDE
	} else if(eventid == g_di_event_right) {
		playback_event(flags | FEV_HOSTONLY, invoker, eventid, delay, origin, angles, fparam1, fparam2, iParam1, iParam2, bParam1, bParam2)
		
		set_weapon_anim(invoker, random_num(8, 12))
		emit_sound(invoker, CHAN_WEAPON, di_sound[0], 1.0, ATTN_NORM, 0, PITCH_NORM)	
		
		static Float:Push[3]
		if(g_shoot_mode[invoker] == MODE_SEMI || g_shoot_mode[invoker] == MODE_RAPID2)
		{
			Push[0] = random_float(-1.0, 1.5)
			Push[1] = random_float(01.5, -1.0)
			Push[2] = 0.0
		} else if(g_shoot_mode[invoker] == MODE_RAPID) {
			Push[0] = random_float(0.0, 0.0)
			Push[1] = random_float(0.0, 0.0)
			Push[2] = 0.0			
		}
			
		set_pev(invoker, pev_punchangle, Push)
		
		return FMRES_SUPERCEDE
	}
	
	return FMRES_HANDLED
}

public fw_SetModel(entity, model[])
{
	if(!is_valid_ent(entity))
		return FMRES_IGNORED
	
	static szClassName[33]
	entity_get_string(entity, EV_SZ_classname, szClassName, charsmax(szClassName))
	
	if(!equal(szClassName, "weaponbox"))
		return FMRES_IGNORED
	
	static iOwner
	iOwner = entity_get_edict(entity, EV_ENT_owner)
	
	if(equal(model, "models/w_elite.mdl"))
	{
		static weapon
		weapon = find_ent_by_owner(-1, weapon_dualinfinity, entity)
		
		if(!is_valid_ent(weapon))
			return FMRES_IGNORED;
		
		if(g_had_dualinfinity[iOwner])
		{
			g_had_dualinfinity[iOwner] = 0
			
			entity_set_int(weapon, EV_INT_impulse, DI_CODE)
			entity_set_model(entity, di_model[3])
			
			return FMRES_SUPERCEDE
		}
	}

	return FMRES_IGNORED;
}

public fw_CmdStart(id, uc_handle, seed)
{
	if(!is_user_alive(id))
		return
	if(get_user_weapon(id) != CSW_DUALINFINITY || !g_had_dualinfinity[id])
		return 

	static CurButton
	CurButton = get_uc(uc_handle, UC_Buttons)
	
	if((CurButton & IN_ATTACK) && !(pev(id, pev_oldbuttons) & IN_ATTACK))
	{
		g_shoot_mode[id] = MODE_SEMI
		fm_get_aim_origin(id, AimOrigin[id])
	} else if(CurButton & IN_ATTACK2) {
		fm_get_aim_origin(id, AimOrigin[id])
		
		if(!(pev(id, pev_oldbuttons) & IN_ATTACK2))
			g_shoot_mode[id] = MODE_RAPID
		else
			g_shoot_mode[id] = MODE_RAPID2
		
		static musket
		musket = find_ent_by_owner(-1, weapon_dualinfinity, id)
		
		if(!pev_valid(musket))
			return	
		if(get_gametime() - 0.1 <= g_last_shoot[id]) 
			return		
			
		ExecuteHamB(Ham_Weapon_PrimaryAttack, musket)
		
		g_last_shoot[id] = get_gametime()
	}
}

public fw_TraceAttack(ent, attacker, Float:Damage, Float:fDir[3], ptr, iDamageType)
{
	if(!is_user_alive(attacker) || !is_user_connected(attacker))
		return HAM_IGNORED	
	if(get_user_weapon(attacker) != CSW_DUALINFINITY || !g_had_dualinfinity[attacker])
		return HAM_IGNORED
		
	static Float:flEnd[3], Float:vecPlane[3]
	
	if(g_shoot_mode[attacker] == MODE_RAPID)
		set_tr2(ptr, TR_vecEndPos, AimOrigin[attacker])
	
	get_tr2(ptr, TR_vecEndPos, flEnd)
	get_tr2(ptr, TR_vecPlaneNormal, vecPlane)		
		
	if(!zbs_is_valid_zombie(ent))
	{
		make_bullet(attacker, flEnd)
		fake_smoke(attacker, ptr)
	} else {
		static Float:flEnd[3]
		get_tr2(ptr, TR_vecEndPos, flEnd)

		create_blood(flEnd)
	}
	
	SetHamParamFloat(3, g_shoot_mode[attacker] == MODE_SEMI ? float(DI_DAMAGE_SEMI) / random_float(1.3, 1.6) : float(DI_DAMAGE_RAPID) / random_float(1.25, 1.6))

	return HAM_HANDLED
}

public fw_AddToPlayer_Post(ent, id)
{
	if(!is_valid_ent(ent))
		return HAM_IGNORED
		
	if(entity_get_int(ent, EV_INT_impulse) == DI_CODE)
	{
		g_had_dualinfinity[id] = 1
		entity_set_int(id, EV_INT_impulse, 0)
		
		return HAM_HANDLED
	}		

	return HAM_HANDLED
}

public fw_PrimaryAttack(ent)
{
	if(!pev_valid(ent))
		return HAM_IGNORED
	
	static id; id = pev(ent,pev_owner)
	if(g_had_dualinfinity[id])
		pev(id, pev_punchangle, cl_pushangle[id])
	
	return HAM_HANDLED	
}

public fw_PrimaryAttack_Post(ent)
{
	if(!pev_valid(ent))
		return HAM_IGNORED
	
	static id; id = pev(ent,pev_owner)
	if(g_had_dualinfinity[id])
	{
		static Float:push[3]
		
		pev(id,pev_punchangle,push)
		xs_vec_sub(push,cl_pushangle[id],push)
		
		if(g_shoot_mode[id] == MODE_SEMI)
			xs_vec_mul_scalar(push, DI_RECOIL_SEMI, push)
		else if(g_shoot_mode[id] == MODE_RAPID2)
			xs_vec_mul_scalar(push, DI_RECOIL_SEMI * 1.75, push)
		else if(g_shoot_mode[id] == DI_RECOIL_RAPID)
			xs_vec_mul_scalar(push, DI_RECOIL_RAPID, push)
		xs_vec_add(push, cl_pushangle[id], push)
		set_pev(id, pev_punchangle, push)	
	}
	
	return HAM_HANDLED	
}


public fw_Reload(ent)
{
	if(!pev_valid(ent))
		return HAM_IGNORED
		
	static id
	id = pev(ent, pev_owner)
	
	if(!is_user_alive(id) || !g_had_dualinfinity[id])
		return HAM_IGNORED		
	
	g_dualinfinity_clip[id] = -1
	
	static bpammo; bpammo = cs_get_user_bpammo(id, CSW_DUALINFINITY)
	if (bpammo <= 0)
		return HAM_SUPERCEDE
	
	new iClip = get_pdata_int(ent, 51, 4)
	if(iClip >= 40)
		return HAM_SUPERCEDE		
	
	g_dualinfinity_clip[id] = iClip

	return HAM_IGNORED
}

public fw_Reload_Post(ent)
{
	if(!pev_valid(ent))
		return HAM_IGNORED
		
	static id
	id = pev(ent, pev_owner)
	
	if(!is_user_alive(id) || !g_had_dualinfinity[id])
		return HAM_IGNORED		
	if(g_dualinfinity_clip[id] == -1)
		return HAM_IGNORED

	set_pdata_float(id, 83, 4.0, 5)
	set_pdata_int(ent, 51, g_dualinfinity_clip[id], 4)
	set_pdata_int(ent, 54, 1, 4)
	
	set_weapon_anim(id, 14)
	
	return HAM_IGNORED
}

public fw_PostFrame(ent)
{
	if(!pev_valid(ent))
		return HAM_IGNORED
		
	static id
	id = pev(ent, pev_owner)
	
	if(!is_user_alive(id) || !g_had_dualinfinity[id])
		return HAM_IGNORED
		
	new Float:flNextAttack = get_pdata_float(id, 83, 5)
	new bpammo = cs_get_user_bpammo(id, CSW_DUALINFINITY)
	
	new iClip = get_pdata_int(ent, 51, 4)
	new fInReload = get_pdata_int(ent, 54, 4)
	
	if(fInReload && flNextAttack <= 0.0)
	{
		new temp = min(40 - iClip, bpammo)
		
		set_pdata_int(ent, 51, iClip + temp, 4)
		cs_set_user_bpammo(id, CSW_DUALINFINITY, bpammo - temp)		
		set_pdata_int(ent, 54, 0, 4)
		
		fInReload = 0
	}		
	
	return HAM_IGNORED
}

/*public fw_Weapon_SendAnim(ent, anim, skip_local)
{
	if(pev_valid(ent) != 2)
		return HAM_IGNORED
		
	new id
	id = get_pdata_cbase(ent, 41 , 4)
	
	if(!g_had_dualinfinity[id])
		return HAM_IGNORED
	
	set_pdata_string(id, m_szAnimExtention * 4, "dualpistols_1", -1 , 20)
	
	return HAM_IGNORED
}*/

public Hook_WeaponHud(id)
{
	engclient_cmd(id, weapon_dualinfinity)
	return PLUGIN_HANDLED
}

public reset_anim(id) g_shoot_mode[id] = 1

public fake_smoke(id, trace_result)
{
	static Float:vecSrc[3], Float:vecEnd[3], TE_FLAG
	
	get_weapon_attachment(id, vecSrc)
	global_get(glb_v_forward, vecEnd)
    
	xs_vec_mul_scalar(vecEnd, 8192.0, vecEnd)
	xs_vec_add(vecSrc, vecEnd, vecEnd)

	get_tr2(trace_result, TR_vecEndPos, vecSrc)
	get_tr2(trace_result, TR_vecPlaneNormal, vecEnd)
    
	xs_vec_mul_scalar(vecEnd, 2.5, vecEnd)
	xs_vec_add(vecSrc, vecEnd, vecEnd)
    
	TE_FLAG |= TE_EXPLFLAG_NODLIGHTS
	TE_FLAG |= TE_EXPLFLAG_NOSOUND
	TE_FLAG |= TE_EXPLFLAG_NOPARTICLES
	
	engfunc(EngFunc_MessageBegin, MSG_PAS, SVC_TEMPENTITY, vecEnd, 0)
	write_byte(TE_EXPLOSION)
	engfunc(EngFunc_WriteCoord, vecEnd[0])
	engfunc(EngFunc_WriteCoord, vecEnd[1])
	engfunc(EngFunc_WriteCoord, vecEnd[2] - 10.0)
	write_short(g_smokepuff_id)
	write_byte(2)
	write_byte(50)
	write_byte(TE_FLAG)
	message_end()
}

/*================================================================================
 [Stocks]
=================================================================================*/

stock set_weapon_anim(id, anim)
{ 
	set_pev(id, pev_weaponanim, anim)
	
	message_begin(MSG_ONE, SVC_WEAPONANIM, _, id)
	write_byte(anim)
	write_byte(pev(id, pev_body))
	message_end()
}

stock make_bullet(id, Float:Origin[3])
{
	// Find target
	new decal = random_num(41, 45)
	const loop_time = 2
	
	static Body, Target
	get_user_aiming(id, Target, Body, 999999)
	
	if(is_user_connected(Target))
		return
	
	for(new i = 0; i < loop_time; i++)
	{
		// Put decal on "world" (a wall)
		message_begin(MSG_BROADCAST, SVC_TEMPENTITY)
		write_byte(TE_WORLDDECAL)
		engfunc(EngFunc_WriteCoord, Origin[0])
		engfunc(EngFunc_WriteCoord, Origin[1])
		engfunc(EngFunc_WriteCoord, Origin[2])
		write_byte(decal)
		message_end()
		
		// Show sparcles
		message_begin(MSG_BROADCAST, SVC_TEMPENTITY)
		write_byte(TE_GUNSHOTDECAL)
		engfunc(EngFunc_WriteCoord, Origin[0])
		engfunc(EngFunc_WriteCoord, Origin[1])
		engfunc(EngFunc_WriteCoord, Origin[2])
		write_short(id)
		write_byte(decal)
		message_end()
	}
}

stock get_weapon_attachment(id, Float:output[3], Float:fDis = 40.0)
{ 
	new Float:vfEnd[3], viEnd[3] 
	get_user_origin(id, viEnd, 3)  
	IVecFVec(viEnd, vfEnd) 
	
	new Float:fOrigin[3], Float:fAngle[3]
	
	pev(id, pev_origin, fOrigin) 
	pev(id, pev_view_ofs, fAngle)
	
	xs_vec_add(fOrigin, fAngle, fOrigin) 
	
	new Float:fAttack[3]
	
	xs_vec_sub(vfEnd, fOrigin, fAttack)
	xs_vec_sub(vfEnd, fOrigin, fAttack) 
	
	new Float:fRate
	
	fRate = fDis / vector_length(fAttack)
	xs_vec_mul_scalar(fAttack, fRate, fAttack)
	
	xs_vec_add(fOrigin, fAttack, output)
}

stock create_blood(const Float:origin[3])
{
	// Show some blood :)
	message_begin(MSG_BROADCAST, SVC_TEMPENTITY) 
	write_byte(TE_BLOODSPRITE)
	engfunc(EngFunc_WriteCoord, origin[0])
	engfunc(EngFunc_WriteCoord, origin[1])
	engfunc(EngFunc_WriteCoord, origin[2])
	write_short(m_iBlood[1])
	write_short(m_iBlood[0])
	write_byte(75)
	write_byte(5)
	message_end()
}

stock set_weapons_timeidle(id, Float:TimeIdle)
{
	if(!is_user_alive(id))
		return
		
	new entwpn = fm_get_user_weapon_entity(id, CSW_DUALINFINITY)
	if(!pev_valid(entwpn)) 
		return
	
	set_pdata_float(entwpn, 46, TimeIdle, 4)
	set_pdata_float(entwpn, 47, TimeIdle, 4)
	set_pdata_float(entwpn, 48, TimeIdle + 1.0, 4)
}

stock set_player_nextattack(id, Float:nexttime)
{
	if(!is_user_alive(id))
		return
		
	const m_flNextAttack = 83
	set_pdata_float(id, m_flNextAttack, nexttime, 5)
}

stock drop_weapons(id, dropwhat)
{
	// Get user weapons
	static weapons[32], num, i, weaponid
	num = 0 // reset passed weapons count (bugfix)
	get_user_weapons(id, weapons, num)
	
	// Loop through them and drop primaries or secondaries
	for (i = 0; i < num; i++)
	{
		// Prevent re-indexing the array
		weaponid = weapons[i]
		
		if ((dropwhat == 1 && ((1<<weaponid) & PRIMARY_WEAPONS_BIT_SUM)) || (dropwhat == 2 && ((1<<weaponid) & SECONDARY_WEAPONS_BIT_SUM)))
		{
			// Get weapon entity
			static wname[32]; get_weaponname(weaponid, wname, charsmax(wname))
			engclient_cmd(id, "drop", wname)
		}
	}
}
