#include <amxmodx>
#include <amxmisc>
#include <engine>
#include <fakemeta>
#include <hamsandwich>
#include <zombie_scenario>

/* Original Venom Guard code by Dias --> Transformed to Zombie Scenario by Nax0ne */

new const zb_model[] = "models/player/zbs_venom_guard/zbs_venom_guard.mdl"
new g_zclass_venomguard, zb_modelindex

new const venomguard_explode_mdl[] = "models/zombie_scenario/ef_boomer.mdl"
new const venomguard_explode_spr[] = "sprites/ZBS/spr_boomer.spr"

new explode_effect_spr;

#define GIB_CLASSNAME "venomguard_gib"

public plugin_init()
{
	register_plugin("[ZBS] Zombie Class: Venom Guard", "1.0", "Nax0ne & Dias");
	
	RegisterHam(Ham_Killed, "info_target", "fw_VenomGuard_Killed", 1);
	register_think(GIB_CLASSNAME, "fw_Think_Gib");
}

public plugin_precache()
{
	zb_modelindex = precache_model(zb_model)
	explode_effect_spr = precache_model(venomguard_explode_spr)
	precache_model(venomguard_explode_mdl)
	
	g_zclass_venomguard = zbs_register_class(600.0, zb_model, zb_modelindex)
}

public fw_VenomGuard_Killed(ent)
{
	if(!is_valid_ent(ent))
		return HAM_IGNORED;
		
	if(!zbs_is_zombie(ent))
		return HAM_IGNORED;
		
	if(!zbs_is_valid_zombie(ent))
		return HAM_IGNORED;
		
	if(zbs_get_class(ent) != g_zclass_venomguard)
		return HAM_IGNORED;		
	
	set_rendering_zbs(ent, 0, 0, 0, 0.0, kRenderFxGlowShell, kRenderTransAlpha)
	Death_Effect(ent)
	
	return HAM_HANDLED;
}

public fw_Think_Gib(ent2)
{
	if(!pev_valid(ent2))
		return;
		
	engfunc(EngFunc_RemoveEntity, ent2)
}

public Death_Effect(ent)
{
	static Float:Origin[3]
	pev(ent, pev_origin, Origin)
	
	Create_Explode_Effect(Origin)
	Create_Explode_Gib(ent)
	
	Check_And_Knockback(ent)
}

public Create_Explode_Effect(Float:Origin[3])
{
	message_begin(MSG_BROADCAST ,SVC_TEMPENTITY)
	write_byte(TE_EXPLOSION)
	engfunc(EngFunc_WriteCoord, Origin[0])
	engfunc(EngFunc_WriteCoord, Origin[1])
	engfunc(EngFunc_WriteCoord, Origin[2] + 15.0)
	write_short(explode_effect_spr)	// sprite index
	write_byte(10)	// scale in 0.1's
	write_byte(15)	// framerate
	write_byte(4)	// flags
	message_end()	
}

public Create_Explode_Gib(owner)
{
	new ent2
	ent2 = engfunc(EngFunc_CreateNamedEntity, engfunc(EngFunc_AllocString, "info_target"))
	
	if(!pev_valid(ent2))
		return
		
	static Float:Origin[3]
	static Float:maxs[3], Float:mins[3]
	
	pev(owner, pev_origin, Origin)
	set_pev(ent2, pev_origin, Origin)
	
	mins[0] = -16.0; maxs[0] = 16.0
	mins[1] = -16.0; maxs[1] = 16.0
	mins[2] = -36.0; maxs[2] = 36.0
	entity_set_size(ent2, mins, maxs)
	engfunc(EngFunc_SetModel, ent2, venomguard_explode_mdl)
	
	set_pev(ent2, pev_classname, GIB_CLASSNAME)
	set_entity_anim(ent2, 0)
	
	set_pev(ent2, pev_nextthink, get_gametime() + 0.25)
}

public Check_And_Knockback(ent)
{
	new Float: iOrigin[3]
	pev(ent, pev_origin, iOrigin)
	
	static const hit_sound[3][] =
	{
		"player/bhit_flesh-1.wav",
		"player/bhit_flesh-2.wav",
		"player/bhit_flesh-3.wav"
	}
	
	static Player, Float: originF[3]
	while((Player = engfunc(EngFunc_FindEntityInSphere, Player, iOrigin, 160.0)))
	{
		if(is_user_alive(Player))
		{
			static Attack_Ent;
			Attack_Ent = find_ent_by_owner(-1, "weapon_knife", Player)
			
			shake_screen(Player)
			
			pev(Player, pev_origin, originF)
			originF[0] = (originF[0] - iOrigin[0]) * 2.0
			originF[1] = (originF[1] - iOrigin[1]) * 2.0
			originF[2] = (originF[2] - iOrigin[2]) + 500.0
				
			set_pev(Player, pev_velocity, originF)
			emit_sound(Player, CHAN_BODY, hit_sound[random(sizeof(hit_sound))], 1.0, ATTN_NORM, 0, PITCH_NORM)
			ExecuteHamB(Ham_TakeDamage, Player,  Attack_Ent, ent, random_float(15.0, 20.0), DMG_BULLET)
		}
	}	
}

stock set_entity_anim(ent, anim)
{
	if(!pev_valid(ent))
		return
	
	entity_set_float(ent, EV_FL_animtime, get_gametime())
	entity_set_float(ent, EV_FL_framerate, 1.0)
	entity_set_float(ent, EV_FL_frame, 0.0)
	
	entity_set_int(ent, EV_INT_sequence, anim)	
}

stock shake_screen(id)
{
	message_begin(MSG_ONE_UNRELIABLE, get_user_msgid("ScreenShake"),{0,0,0}, id)
	write_short(1<<14)
	write_short(1<<13)
	write_short(1<<13)
	message_end()
}

stock set_rendering_zbs(ent, r=255, g=255, b=255, Float:amount=0.0, fx, mode)
{
	if(!pev_valid(ent)) 
		return 0;
		
	static Float:color[3];
	
	color[0]=float(r)
	color[1]=float(g)
	color[2]=float(b)
	
	set_pev(ent,pev_renderfx,fx)
	set_pev(ent,pev_rendercolor,color)
	set_pev(ent,pev_rendermode,mode)
	set_pev(ent,pev_renderamt,amount)
    
	return 1;
}  
